/*
 * LircEmu  -  Remote controller emulation for LIRC
 *
 * Copyright 2002 David Kutalek
 *
 *
 * This file is part of LircEmu.
 * 
 * LircEmu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * LircEmu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with LircEmu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "utils.h"
#include <signal.h>
#include <unistd.h>
#include <curses.h>
#include <stdlib.h>
#include <string.h>

/* key codes used for navigation */
#define KEY_NEXT 9
#define KEY_SELECT 13
#define KEY_EXIT 27

#define MAX_BTNCAP_SIZE 32
#define MAX_FN_LENGTH 1024

#define MAX(x,y) ( ((x) >= (y)) ? x : y )

void init_curses(void) {
	initscr();
	cbreak();
	noecho();
	nonl();
	intrflush(stdscr,FALSE);
	keypad(stdscr,TRUE);
        curs_set(0);
}

/* ----------------------------------------------------------------------------------- */

void finito(void) {
	endwin();
        unlink_socket();
}

struct button {
	int id;
	char caption[MAX_BTNCAP_SIZE];
	char name[MAX_BTNCAP_SIZE];
        char hotkey;
	int x, y;
        struct button *next;
};

/* first, last and selected button from button list */
struct button *first, *last, *selected;

/* width and height of button list */
int width = 0, height = 0;

/* inserts a button to the end of the button list
 * parameter n must be without white space!
 */
void addbtn(int i, char *n, char *c, char hk, int x, int y) {
	struct button *btn;
        int w;

        /* get memory for button */
	if ((btn = (struct button *) malloc(sizeof(*btn))) == NULL) {
		fprintf(stderr, "malloc failed\n");
		exit(1);
	}

        /* set button properties */
	btn->id = i;
	strncpy(btn->caption, c, MAX_BTNCAP_SIZE-1); btn->caption[MAX_BTNCAP_SIZE-1] = 0;
	strncpy(btn->name, n, MAX_BTNCAP_SIZE-1); btn->name[MAX_BTNCAP_SIZE-1] = 0;
        btn->hotkey = hk;
	btn->x = x;
	btn->y = y;

	/* insert it into button list */
	if (first == NULL) {
                /* inserting first button */
		first = btn;
		last = first;
                selected = first;
	}
	else {
		last->next = btn;
		last = btn;
	}

	/* update width & height of button list */
	w = x + strlen(btn->caption);
	width = MAX(width,w);
        height = MAX(height,y);
}

/* inserts all buttons used in this app */
void init_buttons(void) {
	first = NULL;
        addbtn(1,"VISIBLE","Hide/Show",75,0,0);
        addbtn(2,"QUIT","Quit",7,19,0);
        addbtn(3,"PREV","Prev",3,9,3);
        addbtn(4,"MAIN","Main",74,2,3);
        addbtn(5,"BACK","Back",4,2,5);
        addbtn(6,"NEXT","Next",2,9,5);
        addbtn(7,"SELECT","Select",5,16,5);

        addbtn(8,"TV","Tv",'t',0,8);
        addbtn(9,"VIDEO","Video",'v',4,8);
        addbtn(10,"MUSIC","Music",'m',11,8);
        addbtn(11,"UTILS","Utils",'u',18,8);

        addbtn(12,"A","A",'a',1,10);
        addbtn(13,"B","B",'b',5,10);
        addbtn(14,"C","C",'c',9,10);
        addbtn(15,"D","D",'d',13,10);
        addbtn(16,"E","E",'e',17,10);
        addbtn(17,"F","F",'f',21,10);

        addbtn(18,"7","7",'7',5,13);
        addbtn(19,"8","8",'8',11,13);
        addbtn(20,"9","9",'9',17,13);
        addbtn(21,"4","4",'4',5,15);
	addbtn(22,"5","5",'5',11,15);
        addbtn(23,"6","6",'6',17,15);
        addbtn(24,"1","1",'1',5,17);
        addbtn(25,"2","2",'2',11,17);
        addbtn(26,"3","3",'3',17,17);
        addbtn(27,"0","0",'0',11,19);
}

void nextbtn(void) {
	if (selected == NULL) selected=first;
	else if (selected->next == NULL) selected=first;
        else selected=selected->next;
}

void sendbtn(struct button *btn) {
        char line[256];
	sprintf(line, "%016d %d %s %s\n", btn->id, PRESS_LENGTH, btn->name, REMOTE_NAME);
	send_to_all(line);
}

void draw(void) {
	struct button *btn = first;
        int maxx, maxy, top, left, oc;
	char info[16];
	
        /* compute centered position */
	getmaxyx(stdscr,maxy,maxx);
        top = (maxy-height) / 2;
        left = (maxx-width) / 2;

	clear();

	/* draw border */
        attrset(A_NORMAL);
	mvhline(top-2, left-3, '*', width+5);
	mvaddstr(top-2, left+6, "[ LircEmu ]");
	mvvline(top-2, left-3, '*', height+4);
	mvvline(top-2, left+width+2, '*', height+4);
        mvhline(top+height+2, left-3, '*', width+6);
	oc = opened_count();
	snprintf(info, 32, "[ clients: %i/%i ]", oc, MAX_CLIENTS);
	mvaddstr(top+height+2, left+width-20, info);

        /* draw  buttons */
	while (btn != NULL) {
		if (btn == selected) {
			attrset(A_STANDOUT);
		}
		else {
			attrset(A_NORMAL);
		}

		move(top + btn->y, left + btn->x);
		addstr(btn->caption);

                btn = btn->next;
	}

        refresh();
}

void (*oldsh)(int);

void redraw(int foo) {
        oldsh(foo);
	draw();
        signal(SIGWINCH,redraw);
}

/* main loop */
void run(void) {
	char key;
	struct button *btn;
		
	do {
		key = getch();

		switch (key) {
		case KEY_NEXT:
			nextbtn();
                        draw();
			break;
		case KEY_SELECT:
                        sendbtn(selected);
			break;
		default:
			btn = first;
			do {
				if (btn->hotkey == key) {
                                        sendbtn(btn);
					selected = btn;
					draw();
                                        btn = btn->next;
					btn = NULL;
				}
				else btn = btn->next;
			} while (btn != NULL);
                        break;
		}

	} while (key != KEY_EXIT);
}

/* prints some info */
void usage(void) {
	printf("\nLircEmu - a simple Linux Infra Red Controller(LIRC) emulator\nUsage: ");
	printf("lircemu [-s socket_name] [-h]\n\n");
	exit(0);
}

int main(int argc, char **argv){
	char c;
	int pom = 0;
	
	/* process arguments:
	 * -s socket_name - use non-default socket path
	 * -h - get help
	 */
	
	while ((c=getopt(argc,argv,"hs:")) != -1) {
		switch(c) {
			case 'h':
				usage();
				break;
			case 's':
				init_socket(optarg);
				pom = 1;
				break;
			case '?':
				usage();
				break;			
		}
	}

	if (!pom) init_socket("/tmp/lircd");  
		
	/* setup curses, buttons & finito function */
        init_curses();
	init_buttons();
	atexit(finito);
	
	/* setup signal handler for window resizing (when running in xterm...) */
	oldsh = signal(SIGWINCH,redraw);
	
	/* setup drawing when accepted new connection */
	on_connect = draw;
	
        draw();

	run();
	
	return 0;
}
