/*
 * LircEmu  -  Remote controller emulation for LIRC
 *
 * Copyright 2002 David Kutalek
 *
 *
 * This file is part of LircEmu.
 * 
 * LircEmu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * LircEmu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with LircEmu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>

#include "utils.h"

#define NO_CONNECTION -1

/* maximum filename length
 * maybe we should get this from system ??? 
 */
#define MAX_FN_LENGTH 1024

int lirc_socket, lirc_connections[MAX_CLIENTS], oc_count;
char socket_name[MAX_FN_LENGTH];

void sigio_handler(int foo) {
	int i = 0;
	int s;
	
	while ( (i < MAX_CLIENTS) && (lirc_connections[i] != NO_CONNECTION)) i++;

	/* if we accept new connections... */
	if (i < MAX_CLIENTS) {
		if ((s = accept(lirc_socket,NULL,NULL)) == -1) {
			perror("accept connection");
		}
		else {
			lirc_connections[i] = s;
			oc_count++;
			if (on_connect != NULL) on_connect();
		}
	}
	
	/* any other connections are not possible */
	else {
		fprintf(stderr,"Can't accept any other connections (max clients reached)\n");
	}
	
	signal(SIGIO,sigio_handler);
}

void sigpipe_handler(int foo) {	
	signal(SIGPIPE,sigpipe_handler);	
}

/* initialize unix domain socket with path name
 * uses standard lirc socket when name is NULL
 */
int init_socket(char *name) {
	int i;
	struct sockaddr_un sun;
	mode_t mask;

	/* setup connections array */
	for (i=0; i<MAX_CLIENTS; i++) lirc_connections[i] = NO_CONNECTION;
	oc_count = 0;
	on_connect = NULL;
	
	strncpy(socket_name, ( (name==NULL) ? LIRC_STD_SOCKET_NAME : name ), MAX_FN_LENGTH-1);
	socket_name[MAX_FN_LENGTH-1] = 0;

	if ((lirc_socket = socket (AF_UNIX, SOCK_STREAM, 0)) == -1) {
		perror("socket");
		return -1;
	}

	mask = umask(0); /* !!! ??? */
	
	sun.sun_family=AF_UNIX;	
	strcpy(sun.sun_path, socket_name);
	if (bind(lirc_socket, (struct sockaddr *)&sun, sizeof(sun)) == -1) {
		perror("bind");
		umask(mask);
		return -1;
	}
	
	umask(mask);

	if (listen(lirc_socket, MAX_CLIENTS) == -1) {
		perror("listen");
		unlink(socket_name);
		return -1;
	}

	/* register handlers for incoming/closing connections
	 * and make socket asynchronous
	 */
	signal(SIGIO,sigio_handler);
	signal(SIGPIPE,sigpipe_handler);
	fcntl(lirc_socket,F_SETOWN,getpid());
	if (fcntl(lirc_socket,F_SETFL, O_ASYNC) == -1) {
		perror("fcntl, F_SETFL");
		unlink(socket_name);
		return -1;
	}
	
	return 0;
}

/* sends what to all opened connections */
int send_to_all(char *what) {
	int i, len, ec, rv=0;
	char errstr[40];
	for (i=0; i<MAX_CLIENTS; i++) {
		if (lirc_connections[i] != NO_CONNECTION) {
			if ((ec=write(lirc_connections[i], what, len=strlen(what))) != len) {
				/* set no connection on broken pipe */
				if (errno == EPIPE) {
					lirc_connections[i] = NO_CONNECTION;
					oc_count--;
				}
				else {
					snprintf(errstr,39,"Writing to connection number #%d",i); errstr[39] = 0;
					perror(errstr);
					rv = -1;
				}
			}
		}
	}
	
	return rv;
}

/* returns number of opened connections */
int opened_count(void) {
	return oc_count;
}

/* closes all opened connections */
int close_all(void) {
	int i, rv=0;
	char errstr[32];
	for (i=0; i<MAX_CLIENTS; i++) {
		if (lirc_connections[i] != NO_CONNECTION) {
			if (close(lirc_connections[i]) == -1) {
				snprintf(errstr,31,"Closing connection #%d",i); errstr[31] = 0;
				perror(errstr);
				rv = -1;
			}
			lirc_connections[i] = NO_CONNECTION;
		}
	}
	oc_count = 0;
	return rv;
}

/* unlink previously created socket */
int unlink_socket(void) {
	if (unlink(socket_name) == -1) {
		perror("Cannot remove the socket");
		return -1;
	}
	return 0;
}

